xquery version "3.1";

(:~
 : This module provides the API endpoints for retrieving metadata on tei documents.
 :
 : @author Simon Sendler
 : @version 1.0
 : @since 8.1.0
 :)
module namespace kmg-meta="https://mollenhauer-edition.de/ns/kmg-util";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "../conversions/selection.xqm";
import module namespace json = "https://mollenhauer-edition.de/ns/conversions/json" at "../conversions/json.xqm";
import module namespace bibref="https://mollenhauer-edition.de/ns/conversions/bibref" at "../conversions/bibref.xqm";
import module namespace kmg-util="http://mollenhauer-edition.de/ns/util" at "../kmg-util.xqm";
import module namespace rest="http://exquery.org/ns/restxq";
import module namespace functx="http://www.functx.com";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace tg="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace lido="http://www.lido-schema.org";

(:~
 : Returns TextGrid-metadata of the requested document
 :
 : @param $uri A valid TextGrid-URI without prefix
 : @return The requested metadata in JSON
 :)
declare
  %rest:GET
  %rest:path("/api/meta/{$uri}")
  %output:method("json")
function kmg-meta:get-metadata($uri as xs:string)
as map()*
{
    let $doc-path := $config:sade-meta || $uri || ".xml"
    let $doc := doc($doc-path)
    return if (fn:doc-available($doc-path))
        then kmg-meta:format-metadata($doc, $uri)
        else map {"error":"could not find doc with that uri"}
};

(: Formats TextGrid metadata for display via the API.
 :
 : @param $doc A TextGrid metadata-XML
 : @uri A TextGrid URI
 : @return A JSON object
 :)
declare function kmg-meta:format-metadata($doc as node()*, $uri as xs:string)
as map()*
{
    let $title := $doc//tg:title/text() => substring-after("-")
    let $date-created :=
        map {"string": kmg-util:date-to-string($doc//tg:created/text()),
            "raw": $doc//tg:created/text()
        }
    let $date-last-modified :=
        map {"string": kmg-util:date-to-string($doc//tg:lastModified/text()),
            "raw": $doc//tg:lastModified/text()
        }
    let $date-issued :=
        if (fn:exists($doc//tg:issued)) then (
            map {"string": kmg-util:date-to-string($doc//tg:issued/text()),
                "raw": $doc//tg:issued/text()
            }
        )
        else (
            map {"string": "not published",
                "raw": "not published"
            }
        )
    let $tg-uri := $doc//tg:textgridUri/text()
    let $output :=
        map {
            "title": $title,
            "created": $date-created,
            "modified": $date-last-modified,
            "issued": $date-issued,
            "transformed": kmg-util:retrieve-transformation-date($uri),
            "tguri": $tg-uri
        }
    return
        $output
};